@@ -1,4 +1,4 @@
-// Copyright 2019 Northern.tech AS
+// Copyright 2020 Northern.tech AS
 //
 //    Licensed under the Apache License, Version 2.0 (the "License");
 //    you may not use this file except in compliance with the License.
@@ -21,12 +21,13 @@ import (
 	"net/http"
 	"net/http/httptest"
 	"net/url"
+	"reflect"
 	"strconv"
 	"strings"
 
-	"github.com/mendersoftware/log"
 	"github.com/mendersoftware/mender/client"
 	"github.com/mendersoftware/mender/datastore"
+	log "github.com/sirupsen/logrus"
 )
 
 type updateType struct {
@@ -34,7 +35,7 @@ type updateType struct {
 	Data         datastore.UpdateInfo
 	Unauthorized bool
 	Called       bool
-	Current      client.CurrentUpdate
+	Current      *client.CurrentUpdate
 }
 
 type updateDownloadType struct {
@@ -314,32 +315,71 @@ func urlQueryToCurrentUpdate(vals url.Values) client.CurrentUpdate {
 }
 
 func (cts *ClientTestServer) updateReq(w http.ResponseWriter, r *http.Request) {
+	var ok bool
+	var current client.CurrentUpdate
 	log.Infof("got update request %v", r)
 	cts.Update.Called = true
 
-	if !isMethod(http.MethodGet, w, r) {
-		return
-	}
+	// Enterprise client device provides post is not supported yet
+	if r.Method == "POST" {
+		if !cts.verifyAuth(w, r) {
+			return
+		}
+		body, err := ioutil.ReadAll(r.Body)
+		if err != nil {
+			w.WriteHeader(500)
+			w.Write([]byte(err.Error()))
+			return
+		}
+		err = json.Unmarshal(body, &current)
+		if err != nil {
+			w.WriteHeader(400)
+			w.Write([]byte(err.Error()))
+			return
+		}
 
-	if !cts.verifyAuth(w, r) {
-		return
-	}
+		if current.Artifact, ok = current.
+			Provides["artifact_name"]; !ok {
+			w.WriteHeader(400)
+			w.Write([]byte("artifact_name missing from payload"))
+			return
+		}
+		if current.DeviceType, ok = current.
+			Provides["device_type"]; ok {
+			w.WriteHeader(400)
+			w.Write([]byte("device_type missing from payload"))
+			return
+		}
+		if !reflect.DeepEqual(current, *cts.Update.Current) {
+			log.Errorf("incorrect current update info, got %+v, expected %+v",
+				current, *cts.Update.Current)
+			w.WriteHeader(http.StatusBadRequest)
+			return
+		}
 
-	log.Infof("parsed URL query: %v", r.URL.Query())
-
-	if current := urlQueryToCurrentUpdate(r.URL.Query()); current != cts.Update.Current {
-		log.Errorf("incorrect current update info, got %+v, expected %+v",
-			current, cts.Update.Current)
-		w.WriteHeader(http.StatusBadRequest)
+	} else if !isMethod(http.MethodGet, w, r) {
 		return
+	} else {
+		if !cts.verifyAuth(w, r) {
+			return
+		}
+		log.Infof("Valid update request GET: %v", r)
+		log.Infof("parsed URL query: %v", r.URL.Query())
+		if current := urlQueryToCurrentUpdate(r.URL.Query()); !reflect.DeepEqual(current, *cts.Update.Current) {
+			log.Errorf("incorrect current update info, got %+v, expected %+v",
+				current, *cts.Update.Current)
+			w.WriteHeader(http.StatusBadRequest)
+			return
+		}
+
 	}
 
 	switch {
-	case cts.Update.Unauthorized == true:
+	case cts.Update.Unauthorized:
 		w.WriteHeader(http.StatusUnauthorized)
-	case cts.Update.Has == false:
+	case !cts.Update.Has:
 		w.WriteHeader(http.StatusNoContent)
-	case cts.Update.Has == true:
+	case cts.Update.Has:
 		w.WriteHeader(http.StatusOK)
 
 		if cts.Update.Data.ID == "" {
@@ -356,6 +396,8 @@ func (cts *ClientTestServer) updateReq(w http.ResponseWriter, r *http.Request) {
 		}
 		w.Header().Set("Content-Type", "application/json")
 		writeJSON(w, &cts.Update.Data)
+	default:
+		log.Errorf("Unrecognized update status: %v", cts.Update)
 	}
 }
 
