@@ -1,4 +1,4 @@
-// Copyright 2019 Northern.tech AS
+// Copyright 2020 Northern.tech AS
 //
 //    Licensed under the Apache License, Version 2.0 (the "License");
 //    you may not use this file except in compliance with the License.
@@ -20,9 +20,9 @@ import (
 	"os/exec"
 	"strings"
 
-	"github.com/mendersoftware/log"
 	"github.com/mendersoftware/mender/system"
 	"github.com/pkg/errors"
+	log "github.com/sirupsen/logrus"
 )
 
 type UBootEnv struct {
@@ -113,7 +113,7 @@ func (e *UBootEnv) WriteEnv(vars BootVars) error {
 		return err
 	}
 	for k, v := range vars {
-		_, err = fmt.Fprintf(pipe, "%s %s\n", k, v)
+		_, err = fmt.Fprintf(pipe, "%s=%s\n", k, v)
 		if err != nil {
 			log.Error("Error while setting U-Boot variable: ", err)
 			pipe.Close()
