@@ -1,4 +1,4 @@
-// Copyright 2019 Northern.tech AS
+// Copyright 2020 Northern.tech AS
 //
 //    Licensed under the Apache License, Version 2.0 (the "License");
 //    you may not use this file except in compliance with the License.
@@ -19,7 +19,7 @@ import (
 	"os"
 	"path"
 
-	"github.com/mendersoftware/log"
+	log "github.com/sirupsen/logrus"
 )
 
 type DirStore struct {
